@@ -13,6 +13,7 @@
 namespace Composer\Test;
 
 use Composer\Config;
+use Composer\Util\Platform;
 
 class ConfigTest extends TestCase
 {
@@ -327,22 +328,28 @@ class ConfigTest extends TestCase
 
     public function testProcessTimeout()
     {
-        putenv('COMPOSER_PROCESS_TIMEOUT=0');
+        Platform::putEnv('COMPOSER_PROCESS_TIMEOUT', '0');
         $config = new Config(true);
-        $this->assertEquals(0, $config->get('process-timeout'));
-        putenv('COMPOSER_PROCESS_TIMEOUT');
+        $result = $config->get('process-timeout');
+        Platform::clearEnv('COMPOSER_PROCESS_TIMEOUT');
+
+        $this->assertEquals(0, $result);
     }
 
     public function testHtaccessProtect()
     {
-        putenv('COMPOSER_HTACCESS_PROTECT=0');
+        Platform::putEnv('COMPOSER_HTACCESS_PROTECT', '0');
         $config = new Config(true);
-        $this->assertEquals(0, $config->get('htaccess-protect'));
-        putenv('COMPOSER_HTACCESS_PROTECT');
+        $result = $config->get('htaccess-protect');
+        Platform::clearEnv('COMPOSER_HTACCESS_PROTECT');
+
+        $this->assertEquals(0, $result);
     }
 
     public function testGetSourceOfValue()
     {
+        Platform::clearEnv('COMPOSER_PROCESS_TIMEOUT');
+
         $config = new Config;
 
         $this->assertSame(Config::SOURCE_DEFAULT, $config->getSourceOfValue('process-timeout'));
@@ -357,9 +364,11 @@ class ConfigTest extends TestCase
 
     public function testGetSourceOfValueEnvVariables()
     {
-        putenv('COMPOSER_HTACCESS_PROTECT=0');
+        Platform::putEnv('COMPOSER_HTACCESS_PROTECT', '0');
         $config = new Config;
-        $this->assertEquals('COMPOSER_HTACCESS_PROTECT', $config->getSourceOfValue('htaccess-protect'));
-        putenv('COMPOSER_HTACCESS_PROTECT');
+        $result = $config->getSourceOfValue('htaccess-protect');
+        Platform::clearEnv('COMPOSER_HTACCESS_PROTECT');
+
+        $this->assertEquals('COMPOSER_HTACCESS_PROTECT', $result);
     }
 }
